// simple.frag

#extension GL_EXT_gpu_shader4 : enable

// fragment shader of phong shader

#define USE_REFLECTION (1)  /* XyL[vZɔ˃xNggp */

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int lightNumberPhong;
varying vec3 position;
varying vec3 normal;

float GetSpotEffect(int i, float spotAngleCos){
    float spotCosCutoff = gl_LightSource[i].spotCosCutoff;
    float spotEffect = 0.0;

    if (spotCosCutoff < -0.99){
	spotEffect = 1.0;
    }
    else if (spotAngleCos > spotCosCutoff) {
    	spotEffect = 1.0;//pow(spotEffect, gl_LightSource[i].spotExponent);
    }
    else if (spotAngleCos > spotCosCutoff * 0.9) {
	float factor = (spotAngleCos - spotCosCutoff * 0.9) / spotCosCutoff / 0.1;
	spotEffect = -2.0*factor*factor*factor + 3.0*factor*factor;
    }
    else
	spotEffect = 0.0;

    return spotEffect;
}

void FogEffect(){
    if (fogMode == 0) return;

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(gl_Fog.color, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = texture2D(textureImage, gl_TexCoord[0].st);
    if (textureMode == 1){
	texColor[0] += 0.0001;
	texColor[1] += 0.0001;
	texColor[2] += 0.0001;
    }
    else{
	texColor[0] = 1;
	texColor[1] = 1;
	texColor[2] = 1;
	texColor[3] = 1;
    }

    vec4 factor = vec4(1.0, 1.0, 1.0, 0.0); // alpha1𒴂Ȃ悤
    vec3 fnormal = normalize(normal);
    gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (alpha > 1.0) alpha = 1.0;

    for (int i = 0; i < lightNumberPhong; i++){
	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);


	float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				 -lightVec);
	float spotEffect = GetSpotEffect(i, spotAngleCos);

	// W 
	float attenuation = 1.0 / (gl_LightSource[i].constantAttenuation +
				   gl_LightSource[i].linearAttenuation * dis +
				   gl_LightSource[i].quadraticAttenuation * dis * dis);
	//float attenuation = 1.0;
	
	// fBt[Y 
	float diffuse = dot(lightVec, fnormal) * spotEffect * attenuation;
	
	// ArGg 
	gl_FragColor += gl_FrontLightProduct[i].ambient * texColor;
	
	if (diffuse > 0.0){
#if USE_REFLECTION
	    //˃xNg 
	    vec3 viewVec = normalize(-position);
	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = pow(max(dot(viewVec, reflectVec), 0.0),
				 gl_FrontMaterial.shininess) * spotEffect * attenuation;
#else
	    // n[txNg 
	    vec3 viewVec = normalize(-position);
	    vec3 halfVec = normalize(lightVec + viewVec);
	    float specular = pow(max(dot(fnormal, halfVec), 0.0), 
				 gl_FrontMaterial.shininess) * spotEffect * attenuation;
#endif
	    gl_FragColor += 
		gl_FrontLightProduct[i].diffuse * diffuse * texColor * factor
		+ gl_FrontLightProduct[i].specular * specular * factor;
	}

    }

    FogEffect();

    gl_FragColor[3] = alpha;
}
